==> apigw-1.0.3.tgz: package/src/usage-plan.ts <==
import { removeUndefined } from '@tencent-serverless-framework/common';
import { UsagePlanBindSecret } from 'tencentcloud-sdk-nodejs/tencentcloud/services/apigateway/v20180808/apigateway_models';
import { try2Ignore } from 'try2be';
import { Apigw } from '.';
import {
  ApigwBindUsagePlanInputs,
  ApigwRemoveOrUnbindUsagePlanInputs,
  ApigwSetupUsagePlanInputs,
  ApigwSetupUsagePlanSecretInputs,
  ApigwUsagePlan,
} from './interface';

/**
 * 去除数组的重复值
 *
 * @param arr 数组
 * @returns 去除重复值后的数组
 */
export function uniqueArray<T>(arr: T[]) {
  return arr.filter((item, index, self) => {
    return self.indexOf(item) === index;
  });
}

/** API 网关组件用量计划域名子模块 */
export class UsagePlan {
  /** API 网关组件 */
  apigw: Apigw;
  /**
   * 构造函数
   *
   * @param apigw API 网关组件（依赖注入）
   */
  constructor(apigw: Apigw) {
    this.apigw = apigw;
  }
  /**
   * 设置一对 API 密钥
   *
   * @param inputs 输入
   * @returns 密钥 id 列表
   */
  async setupUsagePlanSecret(inputs: ApigwSetupUsagePlanSecretInputs) {
    const { secretName, secretIds, created } = inputs;
    const secretIdsOutput = {
      created: !!created,
      secretIds,
    };

    // user not setup secret ids, just auto generate one
    if (!secretIds || secretIds.length === 0) {
      console.log(`Creating a new Secret key.`);
      const res = await this.apigw.client.CreateApiKey({
        SecretName: secretName,
        AccessKeyType: 'auto',
      });
      const { AccessKeyId, AccessKeySecret } = res.Result!;
      console.log(`Secret id ${AccessKeyId} and key ${AccessKeySecret} created`);
      secretIdsOutput.secretIds = [AccessKeyId];
      secretIdsOutput.created = true;
    } else {
      // use setup secret ids
      // 1. unique it
      // 2. make sure all bind secret ids exist in user's list
      const uniqSecretIds = uniqueArray(secretIds);

      // get all secretId, check local secretId exists
      const res = await this.apigw.client.DescribeApiKeysStatus({
        Limit: uniqSecretIds.length,
        Filters: [
          {
            Name: 'AccessKeyId',
            Values: uniqSecretIds,
          },
        ],
      });
      const { ApiKeySet } = res.Result!;

      const existKeysLen = ApiKeySet.length;

      // Filter invalid and non-existent keys
      const ids: string[] = [];
      uniqSecretIds.forEach((secretId: string) => {
        let found = false;
        let disable = false;
        for (let n = 0; n < existKeysLen; n++) {
          if (ApiKeySet[n] && secretId === ApiKeySet[n].AccessKeyId) {
            if (Number(ApiKeySet[n].Status) === 1) {
              found = true;
            } else {
              disable = true;
              console.log(`There is a disabled secret id ${secretId}, cannot be bound`);
            }
            break;
          }
        }
        if (!found) {
          if (!disable) {
            console.log(`Secret id ${secretId} doesn't exist`);
          }
        } else {
          ids.push(secretId);
        }
      });
      secretIdsOutput.secretIds = ids;
    }

    return secretIdsOutput;
  }

  /**
   * 获取 secrets 列表
   *
   * @param usagePlanId 用户计划 id
   * @param secrets 使用计划绑定密钥数组
   * @param options 获取时的选项
   * @param options.limit 至多获取多少项
   * @param options.offset 跳过多少项开始获取
   * @returns 使用计划 secrets 列表
   */
  async getAllBoundSecrets(
    usagePlanId: string,
    secrets: UsagePlanBindSecret[] = [],
    options: {
      /** 至多获取多少项 */
      limit: number;
      /** 跳过多少项开始获取 */
      offset?: number;
    },
  ): Promise<UsagePlanBindSecret[]> {
    const { limit, offset = 0 } = options;
    const res = await this.apigw.client.DescribeUsagePlanSecretIds({
      UsagePlanId: usagePlanId,
      Limit: limit,
      Offset: offset,
    });
    const { AccessKeyList } = res.Result!;

    if (AccessKeyList.length < limit) {
      return AccessKeyList;
    }
    const more = await this.getAllBoundSecrets(usagePlanId, AccessKeyList, {
      limit,
      offset: offset + AccessKeyList.length,
    });
    // FIXME: more is same type with res, why concat?
    // return res.concat(more.AccessKeyList);
    return secrets.concat(more);
  }

  /**
   * 找到所有未绑定的使用计划 secretIds
   *
   * @param root0 输入参数
   * @param root0.usagePlanId 使用计划 ID
   * @param root0.secretIds 使用计划密钥 ID 列表
   * @returns 找到的不存在的使用计划 id 列表
   */
  async getUnboundSecretIds({
    usagePlanId,
    secretIds,
  }: {
    /** 使用计划 ID */
    usagePlanId: string;
    /** 使用计划密钥 ID 列表 */
    secretIds: string[];
  }) {
    const allBoundSecretObjs = await this.getAllBoundSecrets(usagePlanId, [], { limit: 100 });
    const allBoundSecretIds = allBoundSecretObjs.map((item) => item.AccessKeyId);

    const unboundSecretIds = secretIds.filter((item) => {
      if (allBoundSecretIds.indexOf(item) === -1) {
        return true;
      }
      console.log(`Usage plan ${usagePlanId} secret id ${item} already bound`);
      return false;
    });
    return unboundSecretIds;
  }

  /**
   * 设置 API 网关的使用计划
   *
   * @param inputs API 网关的使用计划输入参数
   * @returns 使用计划的 id
   **/
  async setupUsagePlan(inputs: ApigwSetupUsagePlanInputs) {
    const { usagePlan } = inputs;
    const usageInputs = {
      usagePlanName: usagePlan.usagePlanName ?? '',
      usagePlanDesc: usagePlan.usagePlanDesc ?? '',
      maxRequestNumPreSec: usagePlan.maxRequestNumPreSec ?? -1,
      maxRequestNum: usagePlan.maxRequestNum ?? -1,
    };

    const usagePlanOutput = {
      created: usagePlan.created || false,
      usagePlanId: usagePlan.usagePlanId,
    };

    let exist = false;
    if (usagePlan.usagePlanId) {
      try {
        const res = await this.apigw.client.DescribeUsagePlan({
          UsagePlanId: usagePlan.usagePlanId,
        });
        const detail = res.Result;
        if (detail && detail.UsagePlanId) {
          exist = true;
        }
      } catch (e) {
        // no op
      }
    }

    if (exist) {
      console.log(`Updating usage plan ${usagePlan.usagePlanId}.`);
      await this.apigw.client.ModifyUsagePlan(
        removeUndefined({
          UsagePlanId: usagePlanOutput.usagePlanId,
          UsagePlanName: usageInputs.usagePlanName,
          UsagePlanDesc: usageInputs.usagePlanDesc,
          MaxRequestNumPreSec: usageInputs.maxRequestNumPreSec,
          MaxRequestNum: usageInputs.maxRequestNum,
        }),
      );
    } else {
      const res = await this.apigw.client.CreateUsagePlan(
        removeUndefined({
          UsagePlanName: usageInputs.usagePlanName,
          UsagePlanDesc: usageInputs.usagePlanDesc,
          MaxRequestNumPreSec: usageInputs.maxRequestNumPreSec,
          MaxRequestNum: usageInputs.maxRequestNum,
        }),
      );
      const { UsagePlanId } = res.Result!;

      usagePlanOutput.usagePlanId = UsagePlanId;
      usagePlanOutput.created = true;
      console.log(`Usage plan ${usagePlanOutput.usagePlanId} created.`);
    }

    return usagePlanOutput;
  }

  /**
   * API 网关绑定用量计划
   *
   * @param inputs 输入
   * @returns 输出，绑定后的用量计划
   */
  async bindUsagePlan(inputs: ApigwBindUsagePlanInputs) {
    const { apiId, serviceId, environment, usagePlanConfig, authConfig } = inputs;
    const usagePlan: ApigwUsagePlan = await this.setupUsagePlan({
      usagePlan: usagePlanConfig,
    });

    if (authConfig) {
      const { secretIds = [] } = authConfig;
      const secrets = await this.setupUsagePlanSecret({
        secretName: authConfig.secretName,
        secretIds,
      });

      const unboundSecretIds = await this.getUnboundSecretIds({
        usagePlanId: usagePlan.usagePlanId,
        secretIds: secrets.secretIds!,
      });

      if (unboundSecretIds.length > 0) {
        console.log(
          `Binding secret key ${unboundSecretIds} to usage plan ${usagePlan.usagePlanId}.`,
        );
        await this.apigw.client.BindSecretIds({
          UsagePlanId: usagePlan.usagePlanId,
          AccessKeyIds: unboundSecretIds,
        });
        console.log('Binding secret key successed.');
      }
      // store in api list
      usagePlan.secrets = secrets;
    }

    const res = await this.apigw.client.DescribeApiUsagePlan({
      ServiceId: serviceId!,
      Limit: 100,
    });
    const { ApiUsagePlanList } = res.Result!;

    const oldUsagePlan = ApiUsagePlanList.find((item) => {
      return apiId
        ? item.UsagePlanId === usagePlan.usagePlanId && item.ApiId === apiId
        : item.UsagePlanId === usagePlan.usagePlanId;
    });

    if (oldUsagePlan) {
      if (apiId) {
        console.log(`Usage plan ${usagePlan.usagePlanId} already bind to api ${apiId}`);
      } else {
        console.log(
          `Usage plan ${usagePlan.usagePlanId} already bind to enviromment ${environment}`,
        );
      }

      return usagePlan;
    }

    if (apiId) {
      console.log(`Binding usage plan ${usagePlan.usagePlanId} to api ${apiId}`);
      await this.apigw.client.BindEnvironment({
        ServiceId: serviceId!,
        Environment: environment!,
        BindType: 'API',
        UsagePlanIds: [usagePlan.usagePlanId],
        ApiIds: [apiId],
      });
      console.log(`Bind usage plan ${usagePlan.usagePlanId} to api ${apiId} success`);
      return usagePlan;
    }

    console.log(`Binding usage plan ${usagePlan.usagePlanId} to enviromment ${environment}`);
    await this.apigw.client.BindEnvironment({
      ServiceId: serviceId!,
      Environment: environment!,
      BindType: 'SERVICE',
      UsagePlanIds: [usagePlan.usagePlanId],
    });
    console.log(`Bind usage plan ${usagePlan.usagePlanId} to enviromment ${environment} success`);

    return usagePlan;
  }

  /**
   * 移除 API 网关的使用计划
   *
   * @param inputs 移除 API 网关的使用计划输入参数
   */
  async removeOrUnbindUsagePlan(inputs: ApigwRemoveOrUnbindUsagePlanInputs) {
    const { serviceId, environment, usagePlan, apiId } = inputs;
    // 1.1 unbind secrete ids
    const { secrets } = usagePlan;

    if (secrets && secrets.secretIds) {
      await try2Ignore(async () => {
        return this.apigw.client.UnBindSecretIds({
          AccessKeyIds: secrets.secretIds!,
          UsagePlanId: usagePlan.usagePlanId,
        });
      });
      console.log(`Unbinding secret key from usage plan ${usagePlan.usagePlanId}.`);

      // delelet all created api key
      if (usagePlan.secrets?.created === true) {
        for (let sIdx = 0; sIdx < secrets.secretIds.length; sIdx++) {
          const secretId = secrets.secretIds[sIdx];
          console.log(`Removing secret key ${secretId}`);

          await try2Ignore(async () => {
            return this.apigw.client.DisableApiKey({
              AccessKeyId: secretId,
            });
          });

          await try2Ignore(async () => {
            return this.apigw.client.DeleteApiKey({
              AccessKeyId: secretId,
            });
          });
        }
      }
    }

    // 1.2 unbind environment
    if (apiId) {
      await try2Ignore(async () => {
        return this.apigw.client.UnBindEnvironment({
          ServiceId: serviceId!,
          UsagePlanIds: [usagePlan.usagePlanId],
          Environment: environment,
          BindType: 'API',
          ApiIds: [apiId],
        });
      });
    } else {
      await try2Ignore(async () => {
        return this.apigw.client.UnBindEnvironment({
          ServiceId: serviceId!,
          UsagePlanIds: [usagePlan.usagePlanId],
          Environment: environment,
          BindType: 'SERVICE',
        });
      });
    }

    console.log(`Unbinding usage plan ${usagePlan.usagePlanId} from service ${serviceId}.`);

    // 1.3 delete created usage plan
    if (usagePlan.created === true) {
      console.log(`Removing usage plan ${usagePlan.usagePlanId}`);
      await try2Ignore(async () => {
        return this.apigw.client.DeleteUsagePlan({
          UsagePlanId: usagePlan.usagePlanId,
        });
      });
    }
  }
}

==> apigw-1.1.0.tgz: package/src/usage-plan.ts <==
import { getLogger, printToConsole } from '@jsonlog/logger';
import { removeUndefined } from '@tencent-serverless-framework/common';
import { UsagePlanBindSecret } from 'tencentcloud-sdk-nodejs/tencentcloud/services/apigateway/v20180808/apigateway_models';
import { try2Ignore } from 'try2be';
import { Apigw } from '.';
import {
  ApigwBindUsagePlanInputs,
  ApigwRemoveOrUnbindUsagePlanInputs,
  ApigwSetupUsagePlanInputs,
  ApigwSetupUsagePlanSecretInputs,
  ApigwUsagePlan,
} from './interface';

const logger = getLogger({
  name: 'APIGW',
  middlewares: [printToConsole],
});

/**
 * 去除数组的重复值
 *
 * @param arr 数组
 * @returns 去除重复值后的数组
 */
export function uniqueArray<T>(arr: T[]) {
  return arr.filter((item, index, self) => {
    return self.indexOf(item) === index;
  });
}

/** API 网关组件用量计划域名子模块 */
export class UsagePlan {
  /** API 网关组件 */
  apigw: Apigw;
  /**
   * 构造函数
   *
   * @param apigw API 网关组件（依赖注入）
   */
  constructor(apigw: Apigw) {
    this.apigw = apigw;
  }
  /**
   * 设置一对 API 密钥
   *
   * @param inputs 输入
   * @returns 密钥 id 列表
   */
  async setupUsagePlanSecret(inputs: ApigwSetupUsagePlanSecretInputs) {
    const { secretName, secretIds, created } = inputs;
    const secretIdsOutput = {
      created: !!created,
      secretIds,
    };

    // user not setup secret ids, just auto generate one
    if (!secretIds || secretIds.length === 0) {
      logger.info({ msg: `创建一个新的密钥` });
      const res = await this.apigw.client.CreateApiKey({
        SecretName: secretName,
        AccessKeyType: 'auto',
      });
      const { AccessKeyId, AccessKeySecret } = res.Result!;
      logger.info({ msg: `密钥 ID ${AccessKeyId} 和 Key ${AccessKeySecret} 创建成功` });
      secretIdsOutput.secretIds = [AccessKeyId];
      secretIdsOutput.created = true;
    } else {
      // use setup secret ids
      // 1. unique it
      // 2. make sure all bind secret ids exist in user's list
      const uniqSecretIds = uniqueArray(secretIds);

      // get all secretId, check local secretId exists
      const res = await this.apigw.client.DescribeApiKeysStatus({
        Limit: uniqSecretIds.length,
        Filters: [
          {
            Name: 'AccessKeyId',
            Values: uniqSecretIds,
          },
        ],
      });
      const { ApiKeySet } = res.Result!;

      const existKeysLen = ApiKeySet.length;

      // Filter invalid and non-existent keys
      const ids: string[] = [];
      uniqSecretIds.forEach((secretId: string) => {
        let found = false;
        let disable = false;
        for (let n = 0; n < existKeysLen; n++) {
          if (ApiKeySet[n] && secretId === ApiKeySet[n].AccessKeyId) {
            if (Number(ApiKeySet[n].Status) === 1) {
              found = true;
            } else {
              disable = true;
              logger.info({ msg: `有一个被禁用的 ${secretId} 无法被找到` });
            }
            break;
          }
        }
        if (!found) {
          if (!disable) {
            logger.info({ msg: `密钥 ID ${secretId} 不存在` });
          }
        } else {
          ids.push(secretId);
        }
      });
      secretIdsOutput.secretIds = ids;
    }

    return secretIdsOutput;
  }

  /**
   * 获取 secrets 列表
   *
   * @param usagePlanId 用户计划 id
   * @param secrets 使用计划绑定密钥数组
   * @param options 获取时的选项
   * @param options.limit 至多获取多少项
   * @param options.offset 跳过多少项开始获取
   * @returns 使用计划 secrets 列表
   */
  async getAllBoundSecrets(
    usagePlanId: string,
    secrets: UsagePlanBindSecret[] = [],
    options: {
      /** 至多获取多少项 */
      limit: number;
      /** 跳过多少项开始获取 */
      offset?: number;
    },
  ): Promise<UsagePlanBindSecret[]> {
    const { limit, offset = 0 } = options;
    const res = await this.apigw.client.DescribeUsagePlanSecretIds({
      UsagePlanId: usagePlanId,
      Limit: limit,
      Offset: offset,
    });
    const { AccessKeyList } = res.Result!;

    if (AccessKeyList.length < limit) {
      return AccessKeyList;
    }
    const more = await this.getAllBoundSecrets(usagePlanId, AccessKeyList, {
      limit,
      offset: offset + AccessKeyList.length,
    });
    // FIXME: more is same type with res, why concat?
    // return res.concat(more.AccessKeyList);
    return secrets.concat(more);
  }

  /**
   * 找到所有未绑定的使用计划 secretIds
   *
   * @param root0 输入参数
   * @param root0.usagePlanId 使用计划 ID
   * @param root0.secretIds 使用计划密钥 ID 列表
   * @returns 找到的不存在的使用计划 id 列表
   */
  async getUnboundSecretIds({
    usagePlanId,
    secretIds,
  }: {
    /** 使用计划 ID */
    usagePlanId: string;
    /** 使用计划密钥 ID 列表 */
    secretIds: string[];
  }) {
    const allBoundSecretObjs = await this.getAllBoundSecrets(usagePlanId, [], { limit: 100 });
    const allBoundSecretIds = allBoundSecretObjs.map((item) => item.AccessKeyId);

    const unboundSecretIds = secretIds.filter((item) => {
      if (allBoundSecretIds.indexOf(item) === -1) {
        return true;
      }
      logger.info({ msg: `用户计划 ${usagePlanId} 和密钥 ID ${item} 已经被绑定` });
      return false;
    });
    return unboundSecretIds;
  }

  /**
   * 设置 API 网关的使用计划
   *
   * @param inputs API 网关的使用计划输入参数
   * @returns 使用计划的 id
   **/
  async setupUsagePlan(inputs: ApigwSetupUsagePlanInputs) {
    const { usagePlan } = inputs;
    const usageInputs = {
      usagePlanName: usagePlan.usagePlanName ?? '',
      usagePlanDesc: usagePlan.usagePlanDesc ?? '',
      maxRequestNumPreSec: usagePlan.maxRequestNumPreSec ?? -1,
      maxRequestNum: usagePlan.maxRequestNum ?? -1,
    };

    const usagePlanOutput = {
      created: usagePlan.created || false,
      usagePlanId: usagePlan.usagePlanId,
    };

    let exist = false;
    if (usagePlan.usagePlanId) {
      try {
        const res = await this.apigw.client.DescribeUsagePlan({
          UsagePlanId: usagePlan.usagePlanId,
        });
        const detail = res.Result;
        if (detail && detail.UsagePlanId) {
          exist = true;
        }
      } catch (e) {
        // no op
      }
    }

    if (exist) {
      logger.info({ msg: `正在更新用量计划 ${usagePlan.usagePlanId}.` });
      await this.apigw.client.ModifyUsagePlan(
        removeUndefined({
          UsagePlanId: usagePlanOutput.usagePlanId,
          UsagePlanName: usageInputs.usagePlanName,
          UsagePlanDesc: usageInputs.usagePlanDesc,
          MaxRequestNumPreSec: usageInputs.maxRequestNumPreSec,
          MaxRequestNum: usageInputs.maxRequestNum,
        }),
      );
    } else {
      const res = await this.apigw.client.CreateUsagePlan(
        removeUndefined({
          UsagePlanName: usageInputs.usagePlanName,
          UsagePlanDesc: usageInputs.usagePlanDesc,
          MaxRequestNumPreSec: usageInputs.maxRequestNumPreSec,
          MaxRequestNum: usageInputs.maxRequestNum,
        }),
      );
      const { UsagePlanId } = res.Result!;

      usagePlanOutput.usagePlanId = UsagePlanId;
      usagePlanOutput.created = true;
      logger.info({ msg: `用量计划 ${usagePlanOutput.usagePlanId} 已经创建` });
    }

    return usagePlanOutput;
  }

  /**
   * API 网关绑定用量计划
   *
   * @param inputs 输入
   * @returns 输出，绑定后的用量计划
   */
  async bindUsagePlan(inputs: ApigwBindUsagePlanInputs) {
    const { apiId, serviceId, environment, usagePlanConfig, authConfig } = inputs;
    const usagePlan: ApigwUsagePlan = await this.setupUsagePlan({
      usagePlan: usagePlanConfig,
    });

    if (authConfig) {
      const { secretIds = [] } = authConfig;
      const secrets = await this.setupUsagePlanSecret({
        secretName: authConfig.secretName,
        secretIds,
      });

      const unboundSecretIds = await this.getUnboundSecretIds({
        usagePlanId: usagePlan.usagePlanId,
        secretIds: secrets.secretIds!,
      });

      if (unboundSecretIds.length > 0) {
        logger.info({
          msg: `正在绑定密钥 ${unboundSecretIds} 到用量计划 ${usagePlan.usagePlanId}.`,
        });
        await this.apigw.client.BindSecretIds({
          UsagePlanId: usagePlan.usagePlanId,
          AccessKeyIds: unboundSecretIds,
        });
        logger.info({ msg: '绑定密钥成功' });
      }
      // store in api list
      usagePlan.secrets = secrets;
    }

    const res = await this.apigw.client.DescribeApiUsagePlan({
      ServiceId: serviceId!,
      Limit: 100,
    });
    const { ApiUsagePlanList } = res.Result!;

    const oldUsagePlan = ApiUsagePlanList.find((item) => {
      return apiId
        ? item.UsagePlanId === usagePlan.usagePlanId && item.ApiId === apiId
        : item.UsagePlanId === usagePlan.usagePlanId;
    });

    if (oldUsagePlan) {
      if (apiId) {
        logger.info({ msg: `用量计划 ${usagePlan.usagePlanId} 已经被绑定到 API ${apiId}` });
      } else {
        logger.info({ msg: `用量计划 ${usagePlan.usagePlanId} 已经被绑定到环境 ${environment}` });
      }

      return usagePlan;
    }

    if (apiId) {
      logger.info({ msg: `正在绑定用量计划 ${usagePlan.usagePlanId} 到 API ${apiId}` });
      await this.apigw.client.BindEnvironment({
        ServiceId: serviceId!,
        Environment: environment!,
        BindType: 'API',
        UsagePlanIds: [usagePlan.usagePlanId],
        ApiIds: [apiId],
      });
      logger.info({ msg: `绑定用量计划 ${usagePlan.usagePlanId} 到 API ${apiId} 成功` });
      return usagePlan;
    }

    logger.info({ msg: `正在绑定用量计划 ${usagePlan.usagePlanId} 到环境 ${environment}` });
    await this.apigw.client.BindEnvironment({
      ServiceId: serviceId!,
      Environment: environment!,
      BindType: 'SERVICE',
      UsagePlanIds: [usagePlan.usagePlanId],
    });
    logger.info({ msg: `绑定用量计划 ${usagePlan.usagePlanId} 到环境 ${environment} 成功` });

    return usagePlan;
  }

  /**
   * 移除 API 网关的使用计划
   *
   * @param inputs 移除 API 网关的使用计划输入参数
   */
  async removeOrUnbindUsagePlan(inputs: ApigwRemoveOrUnbindUsagePlanInputs) {
    const { serviceId, environment, usagePlan, apiId } = inputs;
    // 1.1 unbind secrete ids
    const { secrets } = usagePlan;

    if (secrets && secrets.secretIds) {
      await try2Ignore(async () => {
        return this.apigw.client.UnBindSecretIds({
          AccessKeyIds: secrets.secretIds!,
          UsagePlanId: usagePlan.usagePlanId,
        });
      });
      logger.info({ msg: `正在从用量计划 ${usagePlan.usagePlanId} 解绑密钥` });

      // delelet all created api key
      if (usagePlan.secrets?.created === true) {
        for (let sIdx = 0; sIdx < secrets.secretIds.length; sIdx++) {
          const secretId = secrets.secretIds[sIdx];
          logger.info({ msg: `正在移除密钥 ${secretId}` });

          await try2Ignore(async () => {
            return this.apigw.client.DisableApiKey({
              AccessKeyId: secretId,
            });
          });

          await try2Ignore(async () => {
            return this.apigw.client.DeleteApiKey({
              AccessKeyId: secretId,
            });
          });
        }
      }
    }

    // 1.2 unbind environment
    if (apiId) {
      await try2Ignore(async () => {
        return this.apigw.client.UnBindEnvironment({
          ServiceId: serviceId!,
          UsagePlanIds: [usagePlan.usagePlanId],
          Environment: environment,
          BindType: 'API',
          ApiIds: [apiId],
        });
      });
    } else {
      await try2Ignore(async () => {
        return this.apigw.client.UnBindEnvironment({
          ServiceId: serviceId!,
          UsagePlanIds: [usagePlan.usagePlanId],
          Environment: environment,
          BindType: 'SERVICE',
        });
      });
    }

    logger.info({ msg: `正在从服务 ${serviceId} 解绑用量计划 ${usagePlan.usagePlanId}` });

    // 1.3 delete created usage plan
    if (usagePlan.created === true) {
      logger.info({ msg: `正在移除用量计划 ${usagePlan.usagePlanId}` });
      await try2Ignore(async () => {
        return this.apigw.client.DeleteUsagePlan({
          UsagePlanId: usagePlan.usagePlanId,
        });
      });
    }
  }
}
